@@ -1,5 +1,5 @@
 /* BFD back-end data structures for ELF files.
-   Copyright (C) 1992, 93, 94, 95, 96, 1997 Free Software Foundation, Inc.
+   Copyright (C) 1992, 93, 94, 95, 96, 97, 1998 Free Software Foundation, Inc.
    Written by Cygnus Support.
 
 This file is part of BFD, the Binary File Descriptor library.
@@ -32,7 +32,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 #include <sys/dg_xtrace.h>
 #endif /* DGUX */
 
-
 /* If size isn't specified as 64 or 32, NAME macro should fail.  */
 #ifndef NAME
 #if ARCH_SIZE==64
@@ -103,17 +102,24 @@ struct elf_link_hash_entry
   struct elf_link_hash_entry *weakdef;
 
   /* If this symbol requires an entry in the global offset table, the
-     processor specific backend uses this field to hold the offset
-     into the .got section.  If this field is -1, then the symbol does
-     not require a global offset table entry.  */
-  bfd_vma got_offset;
+     processor specific backend uses this field to track usage and
+     final offset.  We use a union and two names primarily to document
+     the intent of any particular piece of code.  The field should be
+     used as a count until size_dynamic_sections, at which point the
+     contents of the .got is fixed.  Afterward, if this field is -1,
+     then the symbol does not require a global offset table entry.  */
+  union
+    {
+      bfd_signed_vma refcount;
+      bfd_vma offset;
+    } got;
 
-  /* If this symbol requires an entry in the procedure linkage table,
-     the processor specific backend uses these two fields to hold the
-     offset into the procedure linkage section and the offset into the
-     .got section.  If plt_offset is -1, then the symbol does not
-     require an entry in the procedure linkage table.  */
-  bfd_vma plt_offset;
+  /* Same, but tracks a procedure linkage table entry.  */
+  union
+    {
+      bfd_signed_vma refcount;
+      bfd_vma offset;
+    } plt;
 
   /* If this symbol is used in the linker created sections, the processor
      specific backend uses this field to map the field into the offset
@@ -133,12 +139,25 @@ struct elf_link_hash_entry
     struct bfd_elf_version_tree *vertree;
   } verinfo;
 
+  /* Virtual table entry use information.  This array is nominally of size
+     size/sizeof(target_void_pointer), though we have to be able to assume
+     and track a size while the symbol is still undefined.  It is indexed
+     via offset/sizeof(target_void_pointer).  */
+  size_t vtable_entries_size;
+  boolean *vtable_entries_used;
+
+  /* Virtual table derivation info.  */
+  struct elf_link_hash_entry *vtable_parent;
+
   /* Symbol type (STT_NOTYPE, STT_OBJECT, etc.).  */
   char type;
 
   /* Symbol st_other value.  */
   unsigned char other;
 
+  /* Hash value of the name computed using the ELF hash function.  */
+  unsigned long elf_hash_value;
+
   /* Some flags; legal values follow.  */
   unsigned short elf_link_hash_flags;
   /* Symbol is referenced by a non-shared object.  */
@@ -161,6 +180,8 @@ struct elf_link_hash_entry
 #define ELF_LINK_HIDDEN 0400
   /* Symbol was forced to local scope due to a version script file.  */
 #define ELF_LINK_FORCED_LOCAL 01000
+  /* Symbol was marked during garbage collection.  */
+#define ELF_LINK_HASH_MARK 02000
 };
 
 /* ELF linker hash table.  */
@@ -303,6 +324,10 @@ struct elf_backend_data
 							  elf_symbol_type *,
 							  unsigned int));
 
+   /* A function to set the type of the info field.  Processor-specific
+     types should be handled here. */
+  int (*elf_backend_get_symbol_type) PARAMS (( Elf_Internal_Sym *, int)); 
+ 
   /* A function to do additional processing on the ELF section header
      just before writing it out.  This is used to set the flags and
      type fields for some sections, or to actually write out data for
@@ -480,6 +505,20 @@ struct elf_backend_data
      backend specific fashion.  */
   boolean (*elf_backend_modify_segment_map) PARAMS ((bfd *));
 
+  /* This function is called during section gc to discover the section a
+     particular relocation refers to.  It need not be defined for hosts
+     that have no queer relocation types.  */
+  asection * (*gc_mark_hook)
+    PARAMS ((bfd *abfd, struct bfd_link_info *, Elf_Internal_Rela *,
+	     struct elf_link_hash_entry *h, Elf_Internal_Sym *));
+
+  /* This function, if defined, is called during the sweep phase of gc
+     in order that a backend might update any data structures it might
+     be maintaining.  */
+  boolean (*gc_sweep_hook)
+    PARAMS ((bfd *abfd, struct bfd_link_info *info, asection *o,
+	     const Elf_Internal_Rela *relocs));
+
   /* The swapping table to use when dealing with ECOFF information.
      Used for the MIPS ELF .mdebug section.  */
   const struct ecoff_debug_swap *elf_backend_ecoff_debug_swap;
@@ -494,11 +533,17 @@ struct elf_backend_data
      .got section */
   bfd_vma got_symbol_offset;
 
+  /* The size in bytes of the headers for the GOT and PLT.  This includes
+     the so-called reserved entries on some systems.  */
+  bfd_vma got_header_size;
+  bfd_vma plt_header_size;
+
   unsigned want_got_plt : 1;
   unsigned plt_readonly : 1;
   unsigned want_plt_sym : 1;
   unsigned plt_not_loaded : 1;
   unsigned plt_alignment : 4;
+  unsigned can_gc_sections : 1;
 };
 
 /* Information stored for each BFD section in an ELF file.  This
@@ -572,7 +617,7 @@ typedef struct elf_linker_section
   bfd_vma max_hole_offset;		/* maximum offset for the hole */
   elf_linker_section_enum_t which;	/* which section this is */
   boolean hole_written_p;		/* whether the hole has been initialized */
-  int alignment;			/* alignment for the section */
+  unsigned int alignment;		/* alignment for the section */
   flagword flags;			/* flags to use to create the section */
 } elf_linker_section_t;
 
@@ -613,15 +658,28 @@ struct elf_obj_tdata
   unsigned int strtab_section, dynsymtab_section;
   unsigned int dynversym_section, dynverdef_section, dynverref_section;
   file_ptr next_file_pos;
+#if 0
+  /* we don't need these inside bfd anymore, and I think
+     these weren't used outside bfd. */
   void *prstatus;			/* The raw /proc prstatus structure */
   void *prpsinfo;			/* The raw /proc prpsinfo structure */
+#endif
+#if defined(DGUX)
   void *cred;
   void *utsname;
   void *dg_pstatus;
   void *dg_lwpstatus;
+#endif /* DGUX */
   bfd_vma gp;				/* The gp value (MIPS only, for now) */
   unsigned int gp_size;			/* The gp size (MIPS only, for now) */
 
+  /* Information grabbed from an elf core file. */
+  int core_signal;
+  int core_pid;
+  int core_lwpid;
+  char* core_program;
+  char* core_command;
+
   /* This is set to true if the object was created by the backend
      linker.  */
   boolean linker;
@@ -632,8 +690,17 @@ struct elf_obj_tdata
   struct elf_link_hash_entry **sym_hashes;
 
   /* A mapping from local symbols to offsets into the global offset
-     table, used when linking.  This is indexed by the symbol index.  */
-  bfd_vma *local_got_offsets;
+     table, used when linking.  This is indexed by the symbol index.
+     Like for the globals, we use a union and two names primarily to
+     document the intent of any particular piece of code.  The field
+     should be used as a count until size_dynamic_sections, at which
+     point the contents of the .got is fixed.  Afterward, if an entry
+     is -1, then the symbol does not require a global offset table entry. */
+  union
+    {
+      bfd_signed_vma *refcounts;
+      bfd_vma *offsets;
+    } local_got;
 
   /* A mapping from local symbols to offsets into the various linker
      sections added.  This is index by the symbol index.  */
@@ -668,6 +735,9 @@ struct elf_obj_tdata
      find_nearest_line.  */
   struct mips_elf_find_line *find_line_info;
 
+  /* A place to stash dwarf1 info for this bfd. */
+  struct dwarf1_debug *dwarf1_find_line_info;
+
   /* A place to stash dwarf2 info for this bfd. */
   struct dwarf2_debug *dwarf2_find_line_info;
 
@@ -680,10 +750,10 @@ struct elf_obj_tdata
   boolean flags_init;
 
   /* Number of symbol version definitions we are about to emit.  */
-  int cverdefs;
+  unsigned int cverdefs;
 
   /* Number of symbol version references we are about to emit.  */
-  int cverrefs;
+  unsigned int cverrefs;
 
   /* Symbol version definitions in external objects.  */
   Elf_Internal_Verdef *verdef;
@@ -716,7 +786,8 @@ struct elf_obj_tdata
 #define elf_gp(bfd)		(elf_tdata(bfd) -> gp)
 #define elf_gp_size(bfd)	(elf_tdata(bfd) -> gp_size)
 #define elf_sym_hashes(bfd)	(elf_tdata(bfd) -> sym_hashes)
-#define elf_local_got_offsets(bfd) (elf_tdata(bfd) -> local_got_offsets)
+#define elf_local_got_refcounts(bfd) (elf_tdata(bfd) -> local_got.refcounts)
+#define elf_local_got_offsets(bfd) (elf_tdata(bfd) -> local_got.offsets)
 #define elf_local_ptr_offsets(bfd) (elf_tdata(bfd) -> linker_section_pointers)
 #define elf_dt_name(bfd)	(elf_tdata(bfd) -> dt_name)
 #define elf_bad_symtab(bfd)	(elf_tdata(bfd) -> bad_symtab)
@@ -768,6 +839,7 @@ extern bfd_reloc_status_type bfd_elf_generic_reloc PARAMS ((bfd *,
 							    bfd *,
 							    char **));
 extern boolean bfd_elf_mkobject PARAMS ((bfd *));
+extern boolean bfd_elf_mkcorefile PARAMS ((bfd *));
 extern Elf_Internal_Shdr *bfd_elf_find_section PARAMS ((bfd *, char *));
 extern boolean _bfd_elf_make_section_from_shdr
   PARAMS ((bfd *abfd, Elf_Internal_Shdr *hdr, const char *name));
@@ -787,6 +859,7 @@ extern boolean _bfd_elf_copy_private_symbol_data
 extern boolean _bfd_elf_copy_private_section_data
   PARAMS ((bfd *, asection *, bfd *, asection *));
 extern boolean _bfd_elf_write_object_contents PARAMS ((bfd *));
+extern boolean _bfd_elf_write_corefile_contents PARAMS ((bfd *));
 extern boolean _bfd_elf_set_section_contents PARAMS ((bfd *, sec_ptr, PTR,
 						       file_ptr,
 						       bfd_size_type));
@@ -989,6 +1062,31 @@ extern Elf_Internal_Rela *_bfd_elf64_link_read_relocs
 #define bfd_elf64_link_record_dynamic_symbol _bfd_elf_link_record_dynamic_symbol
 
 extern boolean _bfd_elf_close_and_cleanup PARAMS ((bfd *));
+extern bfd_reloc_status_type _bfd_elf_rel_vtable_reloc_fn
+  PARAMS ((bfd *, arelent *, struct symbol_cache_entry *, PTR,
+           asection *, bfd *, char **));
+
+boolean _bfd_elf32_gc_sections
+  PARAMS ((bfd *abfd, struct bfd_link_info *info));
+boolean _bfd_elf32_gc_common_finalize_got_offsets
+  PARAMS ((bfd *abfd, struct bfd_link_info *info));
+boolean _bfd_elf32_gc_common_final_link
+  PARAMS ((bfd *, struct bfd_link_info *));
+boolean _bfd_elf32_gc_record_vtinherit
+  PARAMS ((bfd *, asection *, struct elf_link_hash_entry *, bfd_vma));
+boolean _bfd_elf32_gc_record_vtentry
+  PARAMS ((bfd *, asection *, struct elf_link_hash_entry *, bfd_vma));
+
+boolean _bfd_elf64_gc_sections
+  PARAMS ((bfd *abfd, struct bfd_link_info *info));
+boolean _bfd_elf64_gc_common_finalize_got_offsets
+  PARAMS ((bfd *abfd, struct bfd_link_info *info));
+boolean _bfd_elf64_gc_common_final_link
+  PARAMS ((bfd *, struct bfd_link_info *));
+boolean _bfd_elf64_gc_record_vtinherit
+  PARAMS ((bfd *, asection *, struct elf_link_hash_entry *, bfd_vma));
+boolean _bfd_elf64_gc_record_vtentry
+  PARAMS ((bfd *, asection *, struct elf_link_hash_entry *, bfd_vma));
 
 /* MIPS ELF specific routines.  */
 
