@@ -4,6 +4,7 @@
    Department of Pure Mathematics, Cambridge University
    UK , e-mail <takis@dpmms.cam.ac.uk>
 
+   Modified by John Hassey (john_hassey@rtp.dg.com) Nov 1998
 
 This file is part of GDB.
 
@@ -27,6 +28,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 #include "language.h"
 #include "gdbcore.h"
 
+#include <assert.h>
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/dir.h>
@@ -41,10 +43,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 #define user ptrace_user
 
-#ifdef TARGET_HAS_HARDWARE_WATCHPOINTS
-#include <sys/debugreg.h>
-#endif
-
 #include <sys/file.h>
 #include "gdb_stat.h"
 #include "floatformat.h"
@@ -204,16 +202,6 @@ i386dgux_register_u_addr (blockend, regnum)
 /* Wait for child to do something.  Return pid of child, or -1 in case
    of error; store status through argument pointer OURSTATUS.  */
 
-
-static int
-proc_wait_dg (pid, status)
-     int pid;
-     int *status;
-{
-  /* USE wait3 */
-  return wait3(status, 0 ,NULL);
-}
-
 extern void set_sigint_trap();
 extern void set_sigio_trap ();
 extern void clear_sigint_trap();
@@ -370,22 +358,7 @@ i386dgux_extract_return_value(type, regbuf, valbuf)
      char regbuf[REGISTER_BYTES];
      char *valbuf;
 {
-/* On AIX, floating point values are returned in floating point registers.  */
-#ifdef I386_AIX_TARGET
-  if (TYPE_CODE_FLT == TYPE_CODE(type))
-    {
-      double d;
-      /* 387 %st(0), gcc uses this */
-      floatformat_to_double (&floatformat_i387_ext,
-                             &regbuf[REGISTER_BYTE(FP0_REGNUM)],
-                             &d);
-      store_floating (valbuf, TYPE_LENGTH (type), d);
-    }
-  else
-#endif /* I386_AIX_TARGET */
-    {
-      memcpy (valbuf, regbuf, TYPE_LENGTH (type));
-    }
+   memcpy (valbuf, regbuf, TYPE_LENGTH (type));
 }
 
 
@@ -642,15 +615,6 @@ double_to_i387 (from, to)
  *
  */
 
-#if defined(TAKIS_SUPPLY_REGISTER) 
-void
-supply_register (regno, val)
-     int regno;
-     char *val;
-{
-  bcopy (val, &registers[REGISTER_BYTE (regno)], REGISTER_RAW_SIZE (regno));
-}
-#endif /* TAKIS_SUPPLY_REGISTER */
 
 /*
  * We need the following in ix86 DG/ux to be able to handle the code 
@@ -1008,186 +972,85 @@ _i386dgux_initialize_kernel_u_addr ()
 
 #ifdef TARGET_HAS_HARDWARE_WATCHPOINTS
 
-#if !defined (offsetof)
-#define offsetof(TYPE, MEMBER) ((unsigned long) &((TYPE *)0)->MEMBER)
-#endif
-
-/* Record the value of the debug control register.  */
-static int debug_control_mirror;
-
-/* Record which address associates with which register.  */
-static CORE_ADDR address_lookup[DR_LASTADDR - DR_FIRSTADDR + 1];
-
-static int
-i386_insert_aligned_watchpoint PARAMS ((int, CORE_ADDR, CORE_ADDR, int,
-					   int));
-
-static int
-i386_insert_nonaligned_watchpoint PARAMS ((int, CORE_ADDR, CORE_ADDR, int,
-					   int));
-
 /* Insert a watchpoint.  */
 
 int
-i386_insert_watchpoint (pid, addr, len, rw)
+dgux_insert_watchpoint (pid, addr, len, rw)
      int pid;
      CORE_ADDR addr;
      int len;
      int rw;
 {
-  return i386_insert_aligned_watchpoint (pid, addr, addr, len, rw);
+  long watchpoint_id;
+  int status;
+  union dg_xtrace_u u;
+  assert (rw == 0);
+  status = DG_XTRACE_STOP_ON_STORE (pid, (char*)addr, len, &watchpoint_id, &u);
+  if (status == -1)
+    {
+      perror ("dgux_insert_watchpoint()");
+    }
+  return status;
 }
 
-static int
-i386_insert_aligned_watchpoint (pid, waddr, addr, len, rw)
-     int pid;
-     CORE_ADDR waddr;
-     CORE_ADDR addr;
-     int len;
-     int rw;
-{
-  int i;
-  int read_write_bits, len_bits;
-  int free_debug_register;
-  int register_number;
-  
-  /* Look for a free debug register.  */
-  for (i = DR_FIRSTADDR; i <= DR_LASTADDR; i++)
-    {
-      if (address_lookup[i - DR_FIRSTADDR] == 0)
-	break;
-    }
-
-  /* No more debug registers!  */
-  if (i > DR_LASTADDR)
-    return -1;
-
-  read_write_bits = (rw & 1) ? DR_RW_READ : DR_RW_WRITE;
-
-  if (len == 1)
-    len_bits = DR_LEN_1;
-  else if (len == 2)
-    {
-      if (addr % 2)
-	return i386_insert_nonaligned_watchpoint (pid, waddr, addr, len, rw);
-      len_bits = DR_LEN_2;
-    }
-
-  else if (len == 4)
-    {
-      if (addr % 4)
-	return i386_insert_nonaligned_watchpoint (pid, waddr, addr, len, rw);
-      len_bits = DR_LEN_4;
-    }
-  else
-    return i386_insert_nonaligned_watchpoint (pid, waddr, addr, len, rw);
-  
-  free_debug_register = i;
-  register_number = free_debug_register - DR_FIRSTADDR;
-  debug_control_mirror |=
-    ((read_write_bits | len_bits)
-     << (DR_CONTROL_SHIFT + DR_CONTROL_SIZE * register_number));
-  debug_control_mirror |=
-    (1 << (DR_LOCAL_ENABLE_SHIFT + DR_ENABLE_SIZE * register_number));
-  debug_control_mirror |= DR_LOCAL_SLOWDOWN;
-  debug_control_mirror &= ~DR_CONTROL_RESERVED;
-  
-  ptrace (6, pid, offsetof (struct user, u_debugreg[DR_CONTROL]),
-	  debug_control_mirror);
-  ptrace (6, pid, offsetof (struct user, u_debugreg[free_debug_register]),
-	  addr);
-
-  /* Record where we came from.  */
-  address_lookup[register_number] = addr;
-  return 0;
-}
-
-static int
-i386_insert_nonaligned_watchpoint (pid, waddr, addr, len, rw)
-     int pid;
-     CORE_ADDR waddr;
-     CORE_ADDR addr;
-     int len;
-     int rw;
-{
-  int align;
-  int size;
-  int rv;
-
-  static int size_try_array[16] = {
-    1, 1, 1, 1,			/* trying size one */
-    2, 1, 2, 1,			/* trying size two */
-    2, 1, 2, 1,			/* trying size three */
-    4, 1, 2, 1			/* trying size four */
-  };
-
-  rv = 0;
-  while (len > 0)
-    {
-      align = addr % 4;
-      /* Four is the maximum length for 386.  */
-      size = (len > 4) ? 3 : len - 1;
-      size = size_try_array[size * 4 + align];
-
-      rv = i386_insert_aligned_watchpoint (pid, waddr, addr, size, rw);
-      if (rv)
-	{
-	  i386_remove_watchpoint (pid, waddr, size);
-	  return rv;
-	}
-      addr += size;
-      len -= size;
-    }
-  return rv;
-}
 
 /* Remove a watchpoint.  */
 
+/* Code added by Jonh Hassey
+ * <hassey@rtp.dg.com>
+ */
+
 int
-i386_remove_watchpoint (pid, addr, len)
+dgux_remove_watchpoint (pid, addr, len)
      int pid;
      CORE_ADDR addr;
      int len;
 {
-  int i;
-  int register_number;
-
-  for (i = DR_FIRSTADDR; i <= DR_LASTADDR; i++)
-    {
-      register_number = i - DR_FIRSTADDR;
-      if (address_lookup[register_number] == addr)
-	{
-	  debug_control_mirror &=
-	    ~(1 << (DR_LOCAL_ENABLE_SHIFT + DR_ENABLE_SIZE * register_number));
-	  address_lookup[register_number] = 0;
-	}
-    }
-  ptrace (6, pid, offsetof (struct user, u_debugreg[DR_CONTROL]),
-	  debug_control_mirror);
-  ptrace (6, pid, offsetof (struct user, u_debugreg[DR_STATUS]), 0);
-
+  /*
+    FIXME
+    I cheat some and remove them all
+  */
+  union dg_xtrace_u u;
+  DG_XTRACE_REMOVE_ALL_STOP_ON_STORE (pid, &u);
   return 0;
 }
 
+
 /* Check if stopped by a watchpoint.  */
 
 CORE_ADDR
-i386_stopped_by_watchpoint (pid)
+dgux_stopped_by_watchpoint (pid)
     int pid;
 {
-  int i;
+  int result=0;
   int status;
+  int i;
 
-  status = ptrace (3, pid, offsetof (struct user, u_debugreg[DR_STATUS]), 0);
-  ptrace (6, pid, offsetof (struct user, u_debugreg[DR_STATUS]), 0);
+  dg_pstatus_t pstatus;
+  union dg_xtrace_u dgu;
+  long len = sizeof(dg_pstatus_t);
 
-  for (i = DR_FIRSTADDR; i <= DR_LASTADDR; i++)
+  /*
+   * check stop-on-store bit mask in dg extension of
+   * user structure
+   */
+
+  status = DG_XTRACE_READ_TARGET (pid, DG_XT_PSTATUS_SPACE, (char*)0, (char*)&pstatus, &len, &dgu);
+  if (status == -1)
     {
-      if (status & (1 << (i - DR_FIRSTADDR)))
-	return address_lookup[i - DR_FIRSTADDR];
+      perror ("dgux_stopped_by_watchpoint()");
     }
+  else
+    {
+      int i;
 
-  return 0;
+      for(i=0;i<_PT_NUM_STOP_ON_STORE_BIT_MAPS;i++)
+        {
+          if (pstatus.sos_info[i])
+            result = 1;
+        }
+    }
+  return result;
 }
 
 #endif /* TARGET_HAS_HARDWARE_WATCHPOINTS */
